import { LazyResult } from './_reduceLazy';
type FlattenDeep<T> = T extends Array<infer K> ? FlattenDeep2<K> : T;
type FlattenDeep2<T> = T extends Array<infer K> ? FlattenDeep3<K> : T;
type FlattenDeep3<T> = T extends Array<infer K> ? FlattenDeep4<K> : T;
type FlattenDeep4<T> = T extends Array<infer K> ? K : T;
/**
 * Recursively flattens `array`.
 * Note: In `pipe`, use `flattenDeep()` form instead of `flattenDeep`. Otherwise, the inferred type is lost.
 * @param items the target array
 * @signature R.flattenDeep(array)
 * @example
 *    R.flattenDeep([[1, 2], [[3], [4, 5]]]) // => [1, 2, 3, 4, 5]
 *    R.pipe(
 *      [[1, 2], [[3], [4, 5]]],
 *      R.flattenDeep(),
 *    ); // => [1, 2, 3, 4, 5]
 * @category Array
 * @pipeable
 */
export declare function flattenDeep<T>(items: Array<T>): Array<FlattenDeep<T>>;
export declare function flattenDeep<T>(): (items: Array<T>) => Array<FlattenDeep<T>>;
export declare namespace flattenDeep {
    function lazy(): (value: any) => LazyResult<any>;
}
export {};
//# sourceMappingURL=flattenDeep.d.ts.map